LuiginoMPsDialogueSkipper = {}
LuiginoMPsDialogueSkipper.name = "LuiginoMPsDialogueSkipper"
LuiginoMPsDialogueSkipper.savedVariablesName = "LMP_DS_SavedVariables"
local system = LuiginoMPsDialogueSkipper
local stringSeparationCharacter = ","
local anywhere = "Anywhere"
local currentInteractionTarget = nil

local function print(message)
    d(zo_strformat("[<<1>>]: <<2>>.",system.name,message))
end

local function PlayAlertSound()
    PlaySound("Voice_Chat_Alert_Channel_Made_Active")
end

local function PlayErrorSound()
    PlaySound("General_Alert_Error")
end

local function PlayActivatedSound()
    PlaySound("Dialog_Accept")
end

local function PlayDeactivatedSound()
    PlaySound("Dialog_Decline")
end

local function GetSkipAllDialogueStatus()
    local skipAll = LuiginoMPsDialogueSkipper.characterVariables.skipAllDialogue
    if skipAll == true then
        print("Dialogue skipping is currently active")
        PlayActivatedSound()
    elseif skipAll == false then
        print("Dialogue skipping is currently inactive")
        PlayDeactivatedSound()
    else
        print("ERROR! Unknown Dialogue Skip Status")
        PlayErrorSound()
    end
end

local function SetSkipAllDialogueStatus(skipAll)
    LuiginoMPsDialogueSkipper.characterVariables.skipAllDialogue = skipAll
    GetSkipAllDialogueStatus()
end

local function CheckIfSkipAllDialogueStatusExists()
    if LuiginoMPsDialogueSkipper.characterVariables.skipAllDialogue == nil then
        print("Skip All Dialogue Status not found - resorting to default")
        SetSkipAllDialogueStatus(false)
    else
        GetSkipAllDialogueStatus()
    end
end

-- local function GetSkipVendorDialogue()
--     local skipVendor = LuiginoMPsDialogueSkipper.characterVariables.skipVendorDialogue
--     if skipVendor == true then
--         print("Vendor Dialogue skipping is currently active")
--         PlayActivatedSound()
--     elseif skipVendor == false then
--         print("Vendor Dialogue skipping is currently inactive")
--         PlayDeactivatedSound()
--     else
--         print("ERROR! Unknown Vendor Dialogue Skip Status")
--         PlayErrorSound()
--     end
-- end

-- local function SetSkipVendorDialogue(skipVendor)
--     LuiginoMPsDialogueSkipper.characterVariables.skipVendorDialogue = skipVendor
--     GetSkipVendorDialogue()
-- end

-- local function CheckSkipVendorDialogue()
--     if LuiginoMPsDialogueSkipper.characterVariables.skipVendorDialogue == nil then
--         print("Skip Vendor Dialogue Status not found - resorting to default")
--         SetSkipVendorDialogue(false)
--     else
--         GetSkipVendorDialogue()
--     end
-- end

local function GetEntityDialogueWatchListCount()
    local entityCount = 0
    for entity, locations in pairs (LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList) do
        entityCount = entityCount + 1
    end
    if entityCount == 1
    then print("Entity Dialogue Watch List contains 1 entity")
    else print(zo_strformat("Entity Dialogue Watch List contains <<1>> entities",entityCount))
    end
end

local function CreateEntityDialogueWatchList()
    LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList = {}
    GetEntityDialogueWatchListCount()
end

local function CheckIfEntityDialogueWatchListExists()
    if LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList == nil then
        print("No Entity Dialogue Watch List found")
        CreateEntityDialogueWatchList()
    end
end

local function CheckAndAddEntityToList(entityName)
    CheckIfEntityDialogueWatchListExists()
    if LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName] == nil then
        LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName] = {}
        print(zo_strformat("Added entity '<<1>>' to Entiy Dialogue Watch List",entityName))
    end
end

function LuiginoMPsDialogueSkipper.CheckIfEntityLocationExists(entityName, mapName)
    if LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName] == nil then return false
    elseif LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName][anywhere] ~= nil then return true
    elseif LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName][mapName] == nil then return false
    else return true
    end
end

function LuiginoMPsDialogueSkipper.AddToEntityDialogueWatchList(entityName, mapName, skipState)
    CheckAndAddEntityToList(entityName)
    if LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName][mapName] == nil then
        print(zo_strformat("Added location '<<1>>' for entity '<<2>>' to Entiy Dialogue Watch List",mapName,entityName))
    elseif LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName][mapName] ~= skipState then
        print(zo_strformat("Skip State has been updated for entity '<<1>>' in location '<<2>>'",entityName,mapName))
    end
    LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName][mapName] = skipState
    GetEntityDialogueWatchListCount()
    PlayAlertSound()
end

function LuiginoMPsDialogueSkipper.RemoveFromEntityDialogueWatchList(entityName, mapName)
    if entityName == nil then
        PlayErrorSound()
        print("ERROR! Failed to delete entity because no name was given")
    elseif LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName] == nil then
        PlayErrorSound()
        print(zo_strformat("ERROR! Failed to delete entity '<<1>>' because it doesn't exist in the Entity Dialogue Watch List",entityName))
    elseif mapName ~= nil then
        if LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName][mapName] ~= nil then
            LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName][mapName] = nil
            PlayAlertSound()
            print(zo_strformat("Location '<<1>>' for entity '<<2>>' deleted from Entity Dialogue Watch List",mapName,entityName))
        else
            PlayErrorSound()
            print(zo_strformat("ERROR! Failed to delete location '<<1>>' for entity '<<2>>' because it doesn't exist in the Entity Dialogue Watch List",mapName,entityName))
        end
    else
        LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName] = nil
        PlayAlertSound()
        print(zo_strformat("Entity '<<1>>' deleted from Entity Dialogue Watch List",entityName))
        GetEntityDialogueWatchListCount()
    end
end

function LuiginoMPsDialogueSkipper:Initialize()
    LuiginoMPsDialogueSkipper.characterVariables = ZO_SavedVars:NewCharacterIdSettings(system.savedVariablesName, 1, "character", {}, nil)
    LuiginoMPsDialogueSkipper.accountVariables = ZO_SavedVars:NewAccountWide(system.savedVariablesName, 1, "account", {}, nil)
    CheckIfSkipAllDialogueStatusExists()
    -- CheckSkipVendorDialogue()
    CheckIfEntityDialogueWatchListExists()
    GetEntityDialogueWatchListCount()
    print("System initialized")
end

EVENT_MANAGER:RegisterForEvent(system.name, EVENT_ADD_ON_LOADED, function(eventCode, addonName)
    if addonName == system.name then
        system:Initialize()
        EVENT_MANAGER:UnregisterForEvent(system.name, EVENT_ADD_ON_LOADED)
    end
end)

local function Skip()
    local importantOptionPresent = false
    for optionIndex = 1, GetChatterOptionCount() do
        local optionString, optionType, optionalArgument, isImportant, chosenBefore = GetChatterOption(optionIndex)
        if isImportant == true then importantOptionPresent = true end
    end
    if importantOptionPresent == true then
        print("Important option detected")
        PlayAlertSound()
    else
        local optionTaken = false
        for optionIndex = 1, GetChatterOptionCount() do
            local optionString, optionType, optionalArgument, isImportant, chosenBefore = GetChatterOption(optionIndex)
            if chosenBefore == false then
                SelectChatterOption(optionIndex)
                optionTaken = true
                break
            end
        end
        if optionTaken == false then EndInteraction(INTERACTION_CONVERSATION) end
    end
end

local function SetInteractionSkipping(isActive)
    if isActive == true then
        EVENT_MANAGER:RegisterForEvent(system.name, EVENT_CHATTER_BEGIN, function(eventCode, optionCount) Skip() end)
        EVENT_MANAGER:RegisterForEvent(system.name, EVENT_CONVERSATION_UPDATED, function(eventCode, conversationBodyText, conversationOptionCount) Skip() end)
        EVENT_MANAGER:RegisterForEvent(system.name, EVENT_QUEST_OFFERED, function(eventCode) AcceptOfferedQuest() end)
        EVENT_MANAGER:RegisterForEvent(system.name, EVENT_QUEST_COMPLETE_DIALOG, function(eventCode, journalIndex) CompleteQuest() end)
    elseif isActive == false then
        EVENT_MANAGER:UnregisterForEvent(system.name, EVENT_CHATTER_BEGIN)
        EVENT_MANAGER:UnregisterForEvent(system.name, EVENT_CONVERSATION_UPDATED)
        EVENT_MANAGER:UnregisterForEvent(system.name, EVENT_QUEST_OFFERED)
        EVENT_MANAGER:UnregisterForEvent(system.name, EVENT_QUEST_COMPLETE_DIALOG)

    end
end

local function GetEntityWatchListSkipStatus(entityName)
    if LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName] ~= nil then
        local anywhereSkipStatus = LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName][anywhere]
        if anywhereSkipStatus ~= nil then return anywhereSkipStatus
        else
            local currentLocationSkipStatus = LuiginoMPsDialogueSkipper.accountVariables.entityDialogueWatchList[entityName][GetMapName()]
            if currentLocationSkipStatus ~= nil then return currentLocationSkipStatus
            else
                PlayErrorSound()
                print(zo_strformat("ERROR! Could not find skip status for entity '<<1>>' in location '<<2>>'",entityName,GetMapName()))
                return nil
            end
        end
    else return nil
    end
end

EVENT_MANAGER:RegisterForEvent(system.name, EVENT_CLIENT_INTERACT_RESULT, function(eventCode, result, interactTargetName)
    currentInteractionTarget = zo_strformat("<<1>>", interactTargetName)
    local skipEntity = GetEntityWatchListSkipStatus(currentInteractionTarget)
    local skipAll = LuiginoMPsDialogueSkipper.characterVariables.skipAllDialogue
    if skipEntity == nil
    then SetInteractionSkipping(skipAll)
    else SetInteractionSkipping(skipEntity)
    end
end)

EVENT_MANAGER:RegisterForEvent(system.name, EVENT_CHATTER_END, function(eventCode)
    currentInteractionTarget = nil
end)

EVENT_MANAGER:RegisterForEvent(system.name, EVENT_OPEN_STORE, function(eventCode)
    local mapName = GetMapName()
    if currentInteractionTarget ~= nil and system.CheckIfEntityLocationExists(currentInteractionTarget, mapName) == false then
        print(zo_strformat("Vendor '<<1>>' discovered in '<<2>>'",currentInteractionTarget,mapName))
        system.AddToEntityDialogueWatchList(currentInteractionTarget,mapName,true)
    end
end)

SLASH_COMMANDS["/lmpskipallon"] = function() SetSkipAllDialogueStatus(true) end
SLASH_COMMANDS["/lmpskipalloff"] = function() SetSkipAllDialogueStatus(false) end
-- SLASH_COMMANDS["/lmpskipvendoron"] = function() SetSkipVendorDialogue(true) end
-- SLASH_COMMANDS["/lmpskipvendoroff"] = function() SetSkipVendorDialogue(false) end
SLASH_COMMANDS["/lmpskippurge"] = function(inputString)
    if inputString == "CONFIRM" then
        print("ALERT! Entity Dialogue Watch List has been purged")
        CreateEntityDialogueWatchList()
        PlayAlertSound()
    else
        print("WARNING! Purging your Entity Dialogue Watch List is permanent and cannot be reverted! You will lose all NPC data for dialogue skipping. To commit, use the command again and type 'CONFIRM' before entering")
        PlayErrorSound()
    end
end
SLASH_COMMANDS["/lmpskipadd"] = function(inputString)
    local firstSeparatorIndex = string.find(inputString,stringSeparationCharacter)
    local entityName = string.sub(inputString,1,firstSeparatorIndex - 1)
    local secondSeparatorIndex = string.find(inputString,stringSeparationCharacter,firstSeparatorIndex + 1)
    local mapName = string.sub(inputString,firstSeparatorIndex + 1,secondSeparatorIndex - 1)
    if mapName == nil or mapName == "" then mapName = GetMapName() end
    local skipState = string.sub(inputString,secondSeparatorIndex + 1)
    if skipState == "true" then skipState = true
    elseif skipState == "false" then skipState = false
    end
    system.AddToEntityDialogueWatchList(entityName, mapName, skipState)
end
SLASH_COMMANDS["/lmpskipremove"] = function(inputString)
    local entityName = inputString
    local mapName = nil
    local firstSeparatorIndex = string.find(inputString,stringSeparationCharacter)
    if firstSeparatorIndex ~= nil then
        entityName = string.sub(inputString,1,firstSeparatorIndex - 1)
        mapName = string.sub(inputString,firstSeparatorIndex + 1)
        if mapName == "" then mapName = GetMapName() end
    end
    system.RemoveFromEntityDialogueWatchList(entityName, mapName)
end